<?php
namespace Pinpoint\BannerManager\Controller\Adminhtml\Banners;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
/**
 * Class Save
 * @package Pinpoint\BannerManager\Controller\Adminhtml\Banners
 */
class Save extends Action
{
    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Pinpoint\BannerManager\Model\Banner
     */
    protected $banner;

    /**
     * @var \Pinpoint\BannerManager\Model\BannerBlock
     */
    protected $bannerBlock;

    /**
     * Get constructor.
     * @param Context $context
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     */
    public function __construct(
        Context $context,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Pinpoint\BannerManager\Model\Banner $banner,
        \Pinpoint\BannerManager\Model\BannerBlock $bannerBlock
    ) {
        parent::__construct($context);

        $this->resultJsonFactory = $resultJsonFactory;
        $this->banner = $banner;
        $this->bannerBlock = $bannerBlock;
    }

    /**
     * Check the permission to run it
     *
     * @return boolean
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Pinpoint_BannerManager::manage');
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     * @throws \Exception
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Json $resultPage */
        $resultPage = $this->resultJsonFactory->create();

        $banner = $this->banner;

        $bannerData = [
            'banner_name' => $this->getRequest()->getParam('bannerName'),
            'banner_content' => $this->getRequest()->getParam('gridLayout'),
            'enabled' => 1,
            'device_type' => $this->getRequest()->getParam('device_type')
        ];

        if ($bannerId = $this->getRequest()->getParam('id')) {
            $banner = $banner->load($bannerId, 'id');
            $bannerData['id'] = $bannerId;
        }

        $banner->setData($bannerData)->save();

        // load and remove all banner blocks so that they can be re-built
        $blockModel = $this->bannerBlock;
        $allBlocks = $blockModel->getCollection()
            ->addFieldToFilter('banner_id', $bannerId);

        foreach($allBlocks as $block) {
            $block->delete();
        }

        // rebuild all banner blocks
        $params = $this->getRequest()->getParams();
        // reorder so that blocks sit in number order
        ksort($params);
        foreach ($params as $key => $value) {

            if (substr($key, 0, 6) == "block_") {
                $block = $this->bannerBlock;
                $blockData = [
                    'banner_id' => $banner->getId(),
                    'block_ref' => $key,
                    'image' => $this->getBlockValue('block_image_url', $value),
                    'text' => $this->getBlockValue('block_text', $value),
                    'text_colour' => $this->getBlockValue('block_text_colour', $value),
                    'text_position' => $this->getBlockValue('block_text_location', $value),
                    'link_text' => $this->getBlockValue('block_link_text', $value),
                    'link_url' => $this->getBlockValue('block_link_url', $value),
                    'link_target' => $this->getBlockValue('block_link_target', $value),
                    'enabled' => 1,
                    'block_text_size' => $this->getBlockValue('block_text_size', $value),
                    'hide_on_mobile' => $this->getBlockValue('hide_on_mobile', $value),
                    'block_image_location' => $this->getBlockValue('block_image_location', $value)
                ];

                $block->setData($blockData)->save();
            }

        }

        return $resultPage->setData(['success' => true, 'location' => $this->getUrl('*/*/bannerslist')]);
    }

    /**
     * @param $key
     * @param $block
     * @return null
     */
    protected function getBlockValue($key, $block)
    {
        foreach ($block as $input) {
            if ($input['name'] == $key) {
                return $input['value'];
            }
        }

        return null;
    }
}
